@@ -1,8 +1,8 @@
 import logging
 import pathlib
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Optional
 
-from Foundation import NSArray
+from Foundation import NSArray, NSUUID
 from CoreBluetooth import CBPeripheral
 
 from bleak.backends.corebluetooth.CentralManagerDelegate import CentralManagerDelegate
@@ -33,14 +33,14 @@ class BleakScannerCoreBluetooth(BaseBleakScanner):
 
     def __init__(self, **kwargs):
         super(BleakScannerCoreBluetooth, self).__init__(**kwargs)
-        self._identifiers = None
+        self._identifiers: Optional[Dict[NSUUID, Dict[str, Any]]] = None
         self._manager = CentralManagerDelegate.alloc().init()
-        self._timeout = kwargs.get("timeout", 5.0)
+        self._timeout: float = kwargs.get("timeout", 5.0)
 
     async def start(self):
         self._identifiers = {}
 
-        def callback(p: CBPeripheral, a: Dict[str, Any], r: int):
+        def callback(p: CBPeripheral, a: Dict[str, Any], r: int) -> None:
             # update identifiers for scanned device
             self._identifiers.setdefault(p.identifier(), {}).update(a)
 
@@ -63,29 +63,37 @@ class BleakScannerCoreBluetooth(BaseBleakScanner):
                 manufacturer_value = bytes(manufacturer_binary_data[2:])
                 manufacturer_data[manufacturer_id] = manufacturer_value
 
+            service_uuids = [
+                cb_uuid_to_str(u) for u in a.get("kCBAdvDataServiceUUIDs", [])
+            ]
+
             advertisement_data = AdvertisementData(
                 local_name=p.name(),
                 manufacturer_data=manufacturer_data,
                 service_data=service_data,
-                service_uuids=[
-                    cb_uuid_to_str(u) for u in a.get("kCBAdvDataServiceUUIDs", [])
-                ],
+                service_uuids=service_uuids,
                 platform_data=(p, a, r),
             )
 
-            device = BLEDevice(p.identifier().UUIDString(), p.name(), p, r)
+            device = BLEDevice(
+                p.identifier().UUIDString(),
+                p.name(),
+                p,
+                r,
+                uuids=service_uuids,
+                manufacturer_data=manufacturer_data,
+                service_data=service_data,
+                delegate=self._manager.central_manager.delegate(),
+            )
 
             self._callback(device, advertisement_data)
 
         self._manager.callbacks[id(self)] = callback
-        self._manager.start_scan({})
+        await self._manager.start_scan({})
 
     async def stop(self):
-        del self._manager.callbacks[id(self)]
-        try:
-            await self._manager.stop_scan()
-        except Exception as e:
-            logger.warning("stopScan method could not be called: {0}".format(e))
+        await self._manager.stop_scan()
+        self._manager.callbacks.pop(id(self), None)
 
     def set_scanning_filter(self, **kwargs):
         """Set scanning filter for the scanner.
@@ -103,13 +111,14 @@ class BleakScannerCoreBluetooth(BaseBleakScanner):
             "Need to evaluate which macOS versions to support first..."
         )
 
-    async def get_discovered_devices(self) -> List[BLEDevice]:
+    @property
+    def discovered_devices(self) -> List[BLEDevice]:
         found = []
         peripherals = self._manager.central_manager.retrievePeripheralsWithIdentifiers_(
             NSArray(self._identifiers.keys()),
         )
 
-        for i, peripheral in enumerate(peripherals):
+        for peripheral in peripherals:
             address = peripheral.identifier().UUIDString()
             name = peripheral.name() or "Unknown"
             details = peripheral
