@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 
+from typing import Dict
+
+
 uuid16_dict = {
     0x0001: "SDP",
     0x0003: "RFCOMM",
@@ -747,8 +750,8 @@ uuid128_dict = {
     "e95d1b25-251d-470a-a062-fa1922dfa9a8": "MicroBit Temperature Period",
     # Nordic UART Port Emulation */
     "6e400001-b5a3-f393-e0a9-e50e24dcca9e": "Nordic UART Service",
-    "6e400002-b5a3-f393-e0a9-e50e24dcca9e": "Nordic UART TX",
-    "6e400003-b5a3-f393-e0a9-e50e24dcca9e": "Nordic UART RX",
+    "6e400003-b5a3-f393-e0a9-e50e24dcca9e": "Nordic UART TX",
+    "6e400002-b5a3-f393-e0a9-e50e24dcca9e": "Nordic UART RX",
     # from nRF connect
     "be15bee0-6186-407e-8381-0bd89c4d8df4": "Anki Drive Vehicle Service READ",
     "be15bee1-6186-407e-8381-0bd89c4d8df4": "Anki Drive Vehicle Service WRITE",
@@ -863,3 +866,13 @@ def uuidstr_to_str(uuid_):
         return "Unknown"
 
     return s
+
+
+def register_uuids(uuids_to_descriptions: Dict[str, str]) -> None:
+    """Add or modify the mapping of 128-bit UUIDs for services and characteristics to descriptions.
+
+    Args:
+        uuids_to_descriptions: A dictionary of new mappings
+
+    """
+    uuid128_dict.update(uuids_to_descriptions)
